#version 150

#include "/include.glsl"

/* DRAWBUFFERS:0 */
out vec4 outColor0;

uniform sampler2D colortex0;
uniform sampler2D colortex1;

in vec2 texCoord;

vec3 jodieReinhardTonemap(vec3 c){
    float l = dot(c, vec3(0.2126, 0.7152, 0.0722));
    vec3 tc = c / (c + 1.0);

    return mix(c / (l + 1.0), tc, tc);
}

void main() {
    vec3 outColor = texture(colortex0, texCoord).rgb;
    float transparency = texture(colortex0, texCoord).a;

    #ifdef BloomEnabled
        vec3 blurredColor = vec3(0);
        float blurredLuminance = 0.0;
        float totalWeight = 0.0;

        float sigma = BloomBlurRadius / 2.0;         // Standard deviation for the Gaussian
        float twoSigma2 = 2.0 * sigma * sigma;
        float normalizationFactor = 1.0 / (sqrt(twoSigma2 * pi)); // Normalize the kernel

        for (int y  = -BloomBlurRadius; y < BloomBlurRadius; y++) {
            float distanceSquared = y * y;
            float weight = normalizationFactor * exp(-distanceSquared / twoSigma2);

            vec2 offset = vec2(0, y / viewHeight);
            vec2 samplePos = texCoord + offset;
            
            vec4 sampleData = texture(colortex1, samplePos);

            blurredColor += sampleData.rgb * weight;
            blurredLuminance += sampleData.a * weight;
            totalWeight += weight;
        }

        blurredColor /= totalWeight;
        blurredLuminance /= totalWeight;
        
        outColor += blurredColor;

        outColor = jodieReinhardTonemap(outColor);

        outColor *= 1.49992500375;
    #endif

    outColor0 = vec4(outColor, transparency);
}