#version 150

#include "/include.glsl"

/* DRAWBUFFERS:01 */
out vec4 outColor0;
out vec4 outColor1;

uniform sampler2D colortex0;

in vec2 texCoord;

void main() {
    vec3 outColor = texture(colortex0, texCoord).rgb;
    float transparency = texture(colortex0, texCoord).a;

    #ifdef DepthOfFieldEnabled
        vec3 blurredColor = vec3(0);
        float totalWeight = 0.0;

        float sigma = DofBlurRadius / 2.0;         // Standard deviation for the Gaussian
        float twoSigma2 = 2.0 * sigma * sigma;
        float normalizationFactor = 1.0 / (sqrt(twoSigma2 * pi)); // Normalize the kernel

        for (int x = -DofBlurRadius; x < DofBlurRadius; x++) {
            float distanceSquared = x * x;
            float weight = normalizationFactor * exp(-distanceSquared / twoSigma2);

            vec2 offset = vec2(x / viewWidth, 0);
            vec2 samplePos = texCoord + offset;
            
            vec4 sampleData = texture(colortex0, samplePos);

            blurredColor += sampleData.rgb * weight;
            totalWeight += weight;
        }

        outColor1 = vec4(blurredColor / totalWeight, 0);
    #endif

    outColor0 = vec4(outColor, transparency);
}