#version 150

#include "/include.glsl"

/* DRAWBUFFERS:01 */
out vec4 outColor0;
out vec4 outColor1;

uniform sampler2D depthtex0;
uniform sampler2D colortex0;
uniform sampler2D colortex1;

in vec2 texCoord;

void main() {
    vec3 outColor = texture(colortex0, texCoord).rgb;
    float transparency = texture(colortex0, texCoord).a;

    #ifdef DepthOfFieldEnabled
        vec3 position = vec3(texCoord, linearizeDepth(texture(depthtex0, texCoord).r));
        float distanceFromPlayer = position.z;

        vec3 blurredColor = vec3(0);
        float totalWeight = 0.0;

        float sigma = DofBlurRadius / 2.0;         // Standard deviation for the Gaussian
        float twoSigma2 = 2.0 * sigma * sigma;
        float normalizationFactor = 1.0 / (sqrt(twoSigma2 * pi)); // Normalize the kernel

        for (int y = -DofBlurRadius; y < DofBlurRadius; y++) {
            float distanceSquared = y * y;
            float weight = normalizationFactor * exp(-distanceSquared / twoSigma2);

            vec2 offset = vec2(0, y / viewHeight);
            vec2 samplePos = texCoord + offset;
            
            vec4 sampleData = texture(colortex1, samplePos);

            blurredColor += sampleData.rgb * weight;
            totalWeight += weight;
        }
        
        float linearizedCenterDepth = linearizeDepth(centerDepthSmooth);

        float adjustedBlurOffset = DofBlurOffset * linearizedCenterDepth / (far / 20.);

        float blur = smoothstep(adjustedBlurOffset * -((DofBlurSmoothness * 2.0) - 1.0), adjustedBlurOffset, abs(distanceFromPlayer - linearizedCenterDepth));

        outColor = mix(outColor, blurredColor / totalWeight, blur);
    #endif

    #ifdef BloomEnabled
        float luminance = (0.2126 * outColor.r) + (0.7152 * outColor.g) + (0.0722 * outColor.b);
        luminance = step(BloomThreshold, luminance);
        outColor1 = vec4(outColor * luminance, luminance);
    #endif

    outColor0 = vec4(outColor, transparency);
}