#version 150

#include "/include.glsl"

/* DRAWBUFFERS:0 */
out vec4 outColor0;

uniform sampler2D depthtex0;
uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D lightmap;

in vec2 texCoord;

float rand(vec2 co){
    return fract(sin(dot(co, vec2(12.9898, 78.233))) * 43758.5453);
}

vec2 fisheye(vec2 texcoord, float intensity) {
    vec2 fishuv = vec2(0);
    float fishyness = (0.1 + 0.1 * cos(frameTimeCounter * 10.0)) * intensity;
    fishuv.x = (1.0 - texcoord.y*texcoord.y) * fishyness * texcoord.x;
    fishuv.y = (1.0 - texcoord.x*texcoord.x) * fishyness * texcoord.y;
    return fishuv;
}

vec3 chromaticAbberatedColor(sampler2D sample, vec2 fishUv, vec2 texCoord) {
    float chromaticRed = texture(sample, (texCoord + fishUv)).x;
    vec2 chromaticGreenBlue = texture(sample, (texCoord - fishUv)).yz;
    return vec3(chromaticRed, chromaticGreenBlue);
}

void main() {
    vec2 texcoord = texCoord;

    #if View == 0
        vec3 outColor = texture(colortex0, texcoord).rgb; // Default
    #elif View == 1
        vec3 outColor = vec3(texture(colortex0, texcoord).a); // Transperency
    #elif View == 2
        vec3 outColor = vec3((linearizeDepth(texture(depthtex0, texCoord).r) + near) / far); // Depth Buffer (LOVE IT)
    #elif View == 3
        vec3 outColor = texture(colortex1, texCoord).rgb; // Blurred color
    #elif View == 4
        vec3 outColor = vec3(texture(colortex1, texCoord).a); // Luminocity
    #elif View == 5
        vec3 outColor = vec3(texture(colortex2, texCoord).r); // Blocklight
    #elif View == 6
        vec3 outColor = vec3(texture(colortex2, texCoord).g); // Skylight
    #elif View == 7
        vec3 outColor = vec3(length(texture(colortex2, texCoord).rg)); // All light
    #elif View == 8
        vec3 outColor = texture(colortex4, texCoord).rgb; // Light color
    #elif View == 9
        vec3 outColor = texture(colortex3, texCoord).rgb; // Normals
    #elif View == 10
        vec3 outColor = vec3(texture(colortex2, texCoord).ba, texture(colortex3, texCoord).a); // Absolute normals
    #elif View == 11
        vec2 uv = fisheye(texcoord, 5.); // Nausea
        vec3 outColor = chromaticAbberatedColor(colortex0, uv, texcoord);

        float speedScale = 25.0;
        float sizedFrameTimeCouner = frameTimeCounter * speedScale;
        vec3 normal = chromaticAbberatedColor(colortex3, uv, texcoord);

        outColor += abs(normal * vec3(rand((texcoord) + sizedFrameTimeCouner)) + vec3((sin(sizedFrameTimeCouner + rand(texCoord)) * 0.5 + 0.5), (sin(sizedFrameTimeCouner + rand(uv) + ((2.0 * pi) / 3.0)) * 0.5 + 0.5), (sin(sizedFrameTimeCouner + rand(texCoord) + ((4.0 * pi) / 3.0)) * 0.5 + 0.5)));
        outColor /= clamp(sin(frameTimeCounter * 5.0) * 0.5 + 0.5, 1.0, 2.0);
    #endif

    outColor0 = vec4(outColor, 1);
}