#version 150

// Attributes
in vec3 vaPosition;
in vec3 vaNormal;
in vec4 vaColor;
in vec2 vaUV0;
in ivec2 vaUV2;
in vec4 at_tangent;

out vec2 texCoord;
out vec4 shadowUv;
out vec2 screenUv;
out vec3 foliageColor;
out vec2 lightMapCoords;
out float depth;
out float bias;
out vec3 geoNormal;
out vec4 tangent;
out vec3 viewDirection;
out vec3 lightDirection;

#include "/include.glsl"

void main() {
    // Set vertex's texture
    texCoord = vaUV0;

    tangent = vec4(normalize(normalMatrix * at_tangent.rgb), at_tangent.a);
    geoNormal = (normalMatrix * vaNormal);

    vec3 viewPos = modelToView(vaPosition + chunkOffset);
    viewDirection = normalize(cameraPosition - footToWorld(viewToFoot(viewPos)));
    lightDirection = normalize(shadowLightPosition - footToWorld(viewToFoot(viewPos)));

    depth = distance(footToWorld(viewToFoot(viewPos)), cameraPosition) / far;
    
    #ifdef ShadowsEnabled
        vec4 shadowClipPos = shadowViewToClip(footToShadowView(viewToFoot(viewPos) + 0.01 * (mat3(gbufferModelViewInverse) * geoNormal)));

        shadowUv = vec4(shadowClipPos.xyz, clamp(dot(normalize(shadowLightPosition), normalize(geoNormal)), 0, 1));
        float bias = computeBias(shadowClipPos.xyz);
        shadowClipPos.xyz = distort(shadowClipPos.xyz);
        vec3 shadowNdcPos = shadowClipToNdc(shadowClipPos);
        shadowUv.xyz = shadowNdcToScreen(shadowNdcPos);

        vec4 biasOffset = shadowProjection * vec4(mat3(shadowModelView) * (mat3(gbufferModelView) * (geoNormal)), 1);
        shadowUv.xyz += biasOffset.xyz / biasOffset.w * bias;
    #endif

    screenUv = ndcToScreen(viewToNdc(viewPos)).xy;

    // Set foliage color
    foliageColor = vaColor.rgb;

    // Set lighting
    lightMapCoords = vaUV2 * (1.0 / 256.0) + (1.0 / 32.0);

    // Set position of vertex on screen
    gl_Position = viewToClip(viewPos);
}