/* /////////////////////////////////////// */
/* /////////////////////////////////////// */
/*                 Mapping                 */
/* /////////////////////////////////////// */
/* /////////////////////////////////////// */

/*
    outcolor0 == color.rgb, transparency
    outcolor1 == blurColor.rgb, luminocity
    outcolor2 == adjustedBlockLight, adjustedSkyLight, absoluteNormal.xy
    outcolor3 == worldNormal, absoluteNormal.z
    outcolor4 == lightmapColor, 0

*/

/* /////////////////////////////////////// */
/* /////////////////////////////////////// */
/*                 Imports                 */
/* /////////////////////////////////////// */
/* /////////////////////////////////////// */

// Luminosity function: 0.2126*R + 0.7152*G + 0.0722*B

/* Pre-defined */
uniform int isEyeInWater;

uniform float viewHeight;
uniform float viewWidth;
uniform float near;
uniform float far;
uniform float centerDepthSmooth;
uniform float frameTimeCounter;
uniform float alphaTestRef;
uniform float fogDensity;
uniform float sunAngle;

uniform vec3 chunkOffset;
uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform vec3 shadowLightPosition;
uniform vec3 sunPosition;
uniform vec3 upPosition;
uniform vec3 fogColor;

uniform vec4 entityColor;

uniform mat3 normalMatrix;

uniform mat4 modelViewMatrix;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 projectionMatrix;
uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowProjectionInverse;
uniform mat4 gbufferPreviousModelView;
uniform mat4 gbufferPreviousProjection;

/* User settings */

// Shadows
    #define ShadowsEnabled //Enable shadows. Needed for all settings in this tab
    #define ColorShadowsEnabled // Enable colored shadows
    #define ShadowDistortionFactor 0.10 //Fisheye distortion to apply to shadowmap. I would not recommend changing this, any value too small makes shadows wavey, any too big makes them move! [0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.11 0.12 0.13 0.14 0.15 0.16 0.17 0.18 0.19 0.20 0.21 0.22 0.23 0.24 0.25 0.26 0.27 0.28 0.29 0.30 0.31 0.32 0.33 0.34 0.35 0.36 0.37 0.38 0.39 0.40 0.41 0.42 0.43 0.44 0.45 0.46 0.47 0.48 0.49 0.50 0.51 0.52 0.53 0.54 0.55 0.56 0.57 0.58 0.59 0.60 0.61 0.62 0.63 0.64 0.65 0.66 0.67 0.68 0.69 0.70 0.71 0.72 0.73 0.74 0.75 0.76 0.77 0.78 0.79 0.80 0.81 0.82 0.83 0.84 0.85 0.86 0.87 0.88 0.89 0.90 0.91 0.92 0.93 0.94 0.95 0.96 0.97 0.98 0.99 1.00]
    #define ShadowBias 1.00 //Increase this if you get shadow acne. Decrease this if you get peter panning. [0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.60 0.70 0.80 0.90 1.00 1.50 2.00 2.50 3.00 3.50 4.00 4.50 5.00 6.00 7.00 8.00 9.00 10.00]
    #define ShadowBrightness 0.50 //The brightness of shadows. [0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.11 0.12 0.13 0.14 0.15 0.16 0.17 0.18 0.19 0.20 0.21 0.22 0.23 0.24 0.25 0.26 0.27 0.28 0.29 0.30 0.31 0.32 0.33 0.34 0.35 0.36 0.37 0.38 0.39 0.40 0.41 0.42 0.43 0.44 0.45 0.46 0.47 0.48 0.49 0.50 0.51 0.52 0.53 0.54 0.55 0.56 0.57 0.58 0.59 0.60 0.61 0.62 0.63 0.64 0.65 0.66 0.67 0.68 0.69 0.70 0.71 0.72 0.73 0.74 0.75 0.76 0.77 0.78 0.79 0.80 0.81 0.82 0.83 0.84 0.85 0.86 0.87 0.88 0.89 0.90 0.91 0.92 0.93 0.94 0.95 0.96 0.97 0.98 0.99 1.00]
    //#define DarknessPitchBlack //Makes darkness pitch black. Darkens everything slightly.
    #define BlackShadowsAtNight //Makes shadows at night pitch black. Needs darkness pitch black to work.

    const int shadowMapResolution = 2048; //Resolution of shadowmap. Can really impact performance! [216 512 1024 2048 4096 8192]
    const float sunPathRotation = 25; //Rotation offset of sun. [-45 -44 -43 -42 -41 -40 -39 -38 -37 -36 -35 -34 -33 -32 -31 -30 -29 -28 -27 -26 -25 -24 -23 -22 -21 -20 -19 -18 -17 -16 -15 -14 -13 -12 -11 -10 -9 -8 -7 -6 -5 -4 -3 -2 -1 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45]

    const bool shadowtex0Nearest = true;
    const bool shadowtex1Nearest = true;
    const bool shadowcolor0Nearest = true;

// Atmospherics
    // Fog
        #define FogType 1 //What fog type to use. [0 1 2 3 4] // 0 = None, 1 == Smoothstep, 2 = Linear, 3 = Exponential, 4 == Exponential^2
        #define ClipFog //Delete pixels 100% in fog.

        // Smoothstep Settings
            #define SmoothstepFogStartPercent 50 //Start fog percent. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]
            #define SmoothstepFogEndPercent 100 //End fog percent. Anything past this point is in fog. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]

        // Linear Settings
            #define LinearFogStartPercent 25 //Start fog percent. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]
            #define LinearFogEndPercent 75 //End fog percent. Anything past this point is in fog. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]

        // Exponential Settings
            #define ExpDensity 75 //Fog density percent. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]

        // Exponential Squared Settings
            #define Exp2Density 75 //Fog density percent. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100]

// Post Processing
    // Greyscale
        //#define EnableGreyscale //Enable greyscale post-processing.
        #define RemoveGreyscaleFromBlocklight //Whether to remove greyscale with blocklight.
        //#define RemoveGreyscaleFromSkylight //Whether to remove greyscale with skylight.
        #define RemoveGreyscaleAroundPlayer //Whether to remove greyscale in a radius around the player.
        //#define ReverseGreyscale //Whether to reverse the greyscale effects. Anything that removed greyscale now adds it.
        #define GreyscaleDistance 10 //Radius around the player to remove greyscale. [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50]
        #define SkyAlwaysGreyscale //Whether to make the sky greyscale.
        //#define UseGreyscaleInsteadOfLighting //Wether to use greyscale as lighting. Disables normal lighting.

    // DOF
        #define DepthOfFieldEnabled //Enable depth of field.
        #define DofBlurRadius 3 //Radius of box blur. Basically strength of the blur. Can have a performance impact! [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25]
        #define DofBlurOffset 5.0 //How far you can see clearly around your cursor. Scales with distance. [0.0 0.5 1.0 1.5 2.0 2.5 3.0 3.5 4.0 4.5 5.0 5.5 6.0 6.5 7.0 7.5 8.0 8.5 9.0 9.5 10.0 10.5 11.0 11.5 12.0 12.5 13.0 13.5 14.0 14.5 15.0 15.5 16.0 16.5 17.0 17.5 18.0 18.5 19.0 19.5 20.0]
        #define DofBlurSmoothness 0.50 //How smooth the blur is. Percentage of distance from cursor to radius for blur to start. [0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.11 0.12 0.13 0.14 0.15 0.16 0.17 0.18 0.19 0.20 0.21 0.22 0.23 0.24 0.25 0.26 0.27 0.28 0.29 0.30 0.31 0.32 0.33 0.34 0.35 0.36 0.37 0.38 0.39 0.40 0.41 0.42 0.43 0.44 0.45 0.46 0.47 0.48 0.49 0.50 0.51 0.52 0.53 0.54 0.55 0.56 0.57 0.58 0.59 0.60 0.61 0.62 0.63 0.64 0.65 0.66 0.67 0.68 0.69 0.70 0.71 0.72 0.73 0.74 0.75 0.76 0.77 0.78 0.79 0.80 0.81 0.82 0.83 0.84 0.85 0.86 0.87 0.88 0.89 0.90 0.91 0.92 0.93 0.94 0.95 0.96 0.97 0.98 0.99 1.00]
        
        const float centerDepthHalflife = 1.0; //Speed of DOF adjustment. [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0 3.1 3.2 3.3 3.4 3.5 3.6 3.7 3.8 3.9 4.0 4.1 4.2 4.3 4.4 4.5 4.6 4.7 4.8 4.9 5.0]

    // Bloom
        //#define BloomEnabled //Enable bloom. Makes everything look dream-like!
        #define BloomBlurRadius 15 //Basically strength of the blur. Can have a performance impact! [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25]
        #define BloomThreshold 0.75 //Luminance threshold for bloom. [0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.11 0.12 0.13 0.14 0.15 0.16 0.17 0.18 0.19 0.20 0.21 0.22 0.23 0.24 0.25 0.26 0.27 0.28 0.29 0.30 0.31 0.32 0.33 0.34 0.35 0.36 0.37 0.38 0.39 0.40 0.41 0.42 0.43 0.44 0.45 0.46 0.47 0.48 0.49 0.50 0.51 0.52 0.53 0.54 0.55 0.56 0.57 0.58 0.59 0.60 0.61 0.62 0.63 0.64 0.65 0.66 0.67 0.68 0.69 0.70 0.71 0.72 0.73 0.74 0.75 0.76 0.77 0.78 0.79 0.80 0.81 0.82 0.83 0.84 0.85 0.86 0.87 0.88 0.89 0.90 0.91 0.92 0.93 0.94 0.95 0.96 0.97 0.98 0.99 1.00]

// DEBUG
    #define View 0 // Debug view [0 1 2 3 4 5 6 7 8 9 10 11]

/* /////////////////////////////////////// */
/* /////////////////////////////////////// */
/*      Coordinate position switchers      */
/* /////////////////////////////////////// */
/* /////////////////////////////////////// */

float pi = 3.14159265359;
float e = 2.71828182846;

vec3 projectAndDivide(mat4 projectionMatrix, vec3 position) {
    vec4 homogeneousPos = projectionMatrix * vec4(position, 1);
    return homogeneousPos.xyz / homogeneousPos.w;
}

float linearizeDepth(float exponentialDepth) {
    float ndc = exponentialDepth * 2.0 - 1.0;
    return ((2.0 * near * far) / (far + near - ndc * (far - near)));
}

float getLuminosity(vec3 color) {
    return (0.2126 * color.r) + (0.7152 * color.g) + (0.0722 * color.b);
}

mat3 tbnNormalTangent(vec3 normal, vec3 tangent) {
    // For DirectX normal mapping you want to switch the order of these 
    vec3 bitangent = cross(tangent, normal);
    return mat3(tangent, bitangent, normal);
}



vec3 modelToView(vec3 modelPos) {
    return (modelViewMatrix * vec4(modelPos, 1)).xyz;
}

vec4 viewToClip(vec3 viewPos) {
    return gbufferProjection * vec4(viewPos, 1);
}

vec3 clipToView(vec4 clipPos) {
    return (gbufferProjectionInverse * clipPos).xyz;
}

vec3 viewToNdc(vec3 viewPos) {
    return projectAndDivide(gbufferProjection, viewPos);
}

vec3 ndcToView(vec3 ndcPos) {
    return projectAndDivide(gbufferProjectionInverse, ndcPos);
}

vec3 clipToNdc(vec4 clipPos) {
    return clipPos.xyz / clipPos.w;
}

vec3 ndcToScreen(vec3 ndcPos) {
    return ndcPos * 0.5 + 0.5;
}

vec3 screenToNdc(vec3 screenPos) {
    return screenPos * 2.0 - 1.0;
}

vec3 screenToTexel(vec3 screenPos) {
    return screenPos * vec3(viewWidth, viewHeight, 1);
}

vec3 texelToScreen(vec3 texelPos) {
    return texelPos / vec3(viewWidth, viewHeight, 1);
}

vec3 viewToEye(vec3 viewPos) {
    return mat3(gbufferModelViewInverse) * viewPos;
}

vec3 eyeToView(vec3 eyePos) {
    return mat3(gbufferModelView) * eyePos;
}

vec3 viewToFoot(vec3 viewPos) {
    return (gbufferModelViewInverse * vec4(viewPos, 1)).xyz;
}

vec3 footToView(vec3 footPos) {
    return (gbufferModelView * vec4(footPos, 1)).xyz;
}

vec3 footToWorld(vec3 footPos) {
    return footPos + cameraPosition;
}

vec3 worldToFoot(vec3 worldPos) {
    return worldPos - cameraPosition;
}

vec3 footToShadowView(vec3 footPos) {
    return (shadowModelView * vec4(footPos, 1)).xyz;
}

vec3 shadowViewToFoot(vec3 shadowViewPos) {
    return (shadowModelViewInverse * vec4(shadowViewPos, 1)).xyz;
}

vec3 modelToShadowView(vec3 modelPos) {
    return (modelViewMatrix * vec4(modelPos, 1)).xyz;
}

vec4 shadowViewToClip(vec3 shadowViewPos) {
    return shadowProjection * vec4(shadowViewPos, 1);
}

vec3 shadowClipToView(vec4 shadowClipPos) {
    return (shadowProjectionInverse * shadowClipPos).xyz;
}

vec3 shadowClipToNdc(vec4 shadowClipPos) {
    return shadowClipPos.xyz / shadowClipPos.w;
}

vec3 shadowViewToNdc(vec3 shadowViewPos) {
    return projectAndDivide(shadowProjection, shadowViewPos);
}

vec3 shadowNdcToView(vec3 shadowNdcPos) {
    return projectAndDivide(shadowProjectionInverse, shadowNdcPos);
}

vec3 shadowNdcToScreen(vec3 shadowNdcPos) {
    return shadowNdcPos * 0.5 + 0.5;
}

vec3 shadowScreenToNdc(vec3 shadowScreenPos) {
    return shadowScreenPos * 2.0 + 1.0;
}

vec3 shadowScreenToTexel(vec3 shadowScreenPos) {
    return shadowScreenPos * vec3(shadowMapResolution, shadowMapResolution, 1);
}

vec3 shadowTexelToScreen(vec3 shadowTexelPos) {
    return shadowTexelPos / vec3(shadowMapResolution, shadowMapResolution, 1);
}

/* /////////////////////////////////////// */
/* /////////////////////////////////////// */
/*            Shadow distortions           */
/* /////////////////////////////////////// */
/* /////////////////////////////////////// */

vec3 distort(vec3 pos) {
	float factor = length(pos.xy) + ShadowDistortionFactor;
	return vec3(pos.xy / factor, pos.z * 0.5);
}

float computeBias(vec3 pos) {
	float numerator = length(pos.xy) + ShadowDistortionFactor;
	numerator *= numerator;
	return ShadowBias / shadowMapResolution * numerator / ShadowDistortionFactor;
}